import {DrawPile} from "./drawpile.ts";
import {CheckData, CheckDataOption} from "./newmap.ts";
import {getPartLocation, withCamera} from "./layout.ts";
import {Point, Rect, Size} from "./engine/datatypes.ts";
import {D} from "./engine/public.ts";
import {BG_INSET, FG_BOLD} from "./colors.ts";
import {addButton} from "./button.ts";
import {getSkills} from "./skills.ts";
import {getPlayerProgress} from "./playerprogress.ts";

export class CheckModal {
  #drawpile: DrawPile;
  #activeCheck: CheckData | null;
  #callback: (() => void) | null;

  constructor() {
    this.#drawpile = new DrawPile();
    this.#activeCheck = null;
    this.#callback = null;
  }

  get isShown() {
    return this.#activeCheck != null
  }

  get #size(): Size {
    return getPartLocation("BottomModal").size
  }

  update() {
    withCamera("BottomModal", () => this.#update())
    this.#drawpile.executeOnClick()
  }

  draw() {
    withCamera("BottomModal", () => this.#draw())
  }

  show(checkData: CheckData | null, callback: (() => void) | null) {
    this.#activeCheck = checkData;
    this.#callback = callback;
  }

  #update() {
    this.#drawpile.clear();

    let check = this.#activeCheck
    if (!check) { return; }

    let size = this.#size;
    this.#drawpile.add(0, () => {
      D.fillRect(new Point(-4, -4), size.add(new Size(8, 8)), BG_INSET)
    })

    let labelText = check.label;
    this.#drawpile.add(0, () => {
      D.drawText(labelText, new Point(0, 0), FG_BOLD, {
        forceWidth: size.w
      })
    })

    let options = check.options;

    let addOptionButton = (option: CheckDataOption, rect: Rect) => {
      let skill = option.skill();
      let skillName = getSkills().get(skill).profile.name;
      let hasSkill = getPlayerProgress().hasLearned(skill);
      hasSkill ||= true;
      let optionLabel: string
      if (hasSkill) {
        optionLabel = `[${skillName}] ${option.unlockable}`;
      } else {
        optionLabel = `[Needs ${skillName}] ${option.locked}`;
      }
      addButton(this.#drawpile, optionLabel, rect, hasSkill, () => {
        let cb = this.#callback;
        if (cb) { cb(); }
        this.show(null, null);
      })
    }

    if (options.length == 1) {
      addOptionButton(options[0], new Rect(new Point(0, size.h - 64), new Size(size.w, 64)));
    }
    else if (options.length == 2) {
      addOptionButton(options[0], new Rect(new Point(0, size.h - 64), new Size(size.w, 32)));
      addOptionButton(options[1], new Rect(new Point(0, size.h - 32), new Size(size.w, 32)));
    } else {
      throw new Error(`unexpected number of options ${options.length}`)
    }
  }

  #draw() {
    this.#drawpile.draw();
  }
}

let active: CheckModal = new CheckModal();
export function getCheckModal() {
  return active;
}